// =======================================================
// ====== Programm zur Erstellung eines Wärmebilds =======
// =======================================================


#include <Servo.h>           // Inkludiert die Servodateien 
#include <i2cmaster.h>       // Inkludiert die i2c-Bus-Dateien
 
Servo servo_horiz;           // Definiert den horizontalen servo
Servo servo_verti;           // Definiert den vertikalen servo
int taste;                   // Tastaturpin für Start des Scans
int horiz_start = 90;        // horizontaler Startwinkel
int verti_start = 20;        // vertikaler Startwinkel
int schritte = 40;           // horizontale bzw. vertikale Schritte


// ************************************
// ************** SETUP ***************
// ************************************

void setup() 
   { 
    Serial.begin(9600);
     
    servo_horiz.attach(9);    // Set horizontalen servo to digital pin 9
    servo_verti.attach(10);   // Set vertikalen servo to digital pin 10
    
    pinMode(13, OUTPUT);
    
    
    i2c_init();                              //Initialisiert den i2c-Bus
    PORTC = (1 << PORTC4) | (1 << PORTC5);   //enable pullups
    
   } 


// ********************************************
// ************** HAUPTSCHLEIFE ***************
// ********************************************
 
void loop() 
   { 
  
    taste = analogRead(A0);      // Abfrage, ob die Taste gedrückt wird und der Scan gestartet werden kann
    
    servo_horiz.write(horiz_start);
    servo_verti.write(verti_start);
    
    if (taste > 500)
       {        
        // ********** Messung gestartet ************ 
          
   
        digitalWrite(13, HIGH);   // set the LED on
        delay(1000);              // wait for a second
        digitalWrite(13, LOW);    // set the LED off
    
         
        for (int i = horiz_start; i >= horiz_start - schritte; i--)
          {
           servo_verti.write(verti_start);
                     
           delay(150);
                
           for (int j = verti_start; j <= verti_start + schritte; j++)
              { 
               servo_horiz.write(i);
               servo_verti.write(j);
           
               delay(250);
               
               
               int dev = 0x5A<<1;
               int data_low = 0;
               int data_high = 0;
               int pec = 0;
    
               i2c_start_wait(dev+I2C_WRITE);
               i2c_write(0x07);
    
               // Temperatur vom MLX90614 lesen
        
               i2c_rep_start(dev+I2C_READ);
               data_low = i2c_readAck();      // Read 1 byte and then send back
               data_high = i2c_readAck();     // Read 1 byte and then send back
               pec = i2c_readNak();
               i2c_stop();
    
               // This converts high and low bytes together and processes temperature, MSB is a error bit and is ignored for temps
        
               double tempFactor = 0.02;  // 0.02 degrees per LSB (measurement resolution of the MLX90614)
               double tempData = 0x0000;  // zero out the data
               int frac;                  // data past the decimal point
    
               // This masks off the error bit of the high byte, then moves it left 8 bits and adds the low byte.
        
               tempData = (double)(((data_high & 0x007F) << 8) + data_low);
               tempData = (tempData * tempFactor)-0.01;
    
               float celcius = tempData - 273.15;
    
               Serial.print(horiz_start - i);
               Serial.print(" ");
               Serial.print(j - verti_start);
               Serial.print(" ");
               Serial.println(celcius);
                  
              }           
          }
     
        // ********** Messung beendet ************  
          
        digitalWrite(13, HIGH);   // set the LED on
        delay(1000);              // wait for a second
        digitalWrite(13, LOW);    // set the LED off
        
        servo_horiz.write(horiz_start);
        servo_verti.write(verti_start);        
       }
         
   } 
